'******************************************************************************
' This is File fb-Btest.bas - FreeBASIC version of dplotJR's Btest.bas
'
' Date: 25 Nov 2005
'
' Purpose:
'  To supply example code for FreeBASIC to DPLOTJR
'
' Author(s):
'  Translated (with changes) from supplied source by
'  T. McCloskey, Clifton Park, NY 12065
'
' Translated (with changes) from VB source "Btest.frm" with changes.
'
' Notes:
'   1) FreeBASIC stores strings with trailing nulls much like "C" . Be aware,
'      however, that a fixed length string of length "n" can hold "n-1" bytes
'      of user data and FreeBASIC reserves the "n-th" byte for the trailing null.
'
'   2) FreeBASIC stores array data contigeous, but in ROW Major order, much
'      like "C" . Hence, multi-dimensioned array examples from VB and other
'      BASIC or FORTRAN source need to adjust indicies. See below for some
'      examples. This file has a static DEFINE variable ("COLMAJ") that can
'      be defined to select the COLUMN major storage scheme (for use with
'      other than FreeBASIC) if the user desires.
'
'   3) This file allows two possible ways (include files) to resolve the external
'      calls to the Dynamic Link Library "DPLOTLIB.DLL".
'
'        - "fb-dplotlib.bi" - The first alternative is to resolve via the link
'          process utilizing an interfaace (import) file. This import file can
'          be constructed using a tool supplied via FreeBASIC (DLLTOOL.EXE)
'          and the DLL "DEF" file specifically constructed for this library
'          for this purpose (see "DPLOTLIB.DLL.DEF" enclosed separately or at
'          tail of file "fb-dplotlib.bi"). Following typical import library naming
'          conventions ("lib<name>.a"), this import file would usually be 
'          named "libDPLOTLIB.DLL.a". Once this file is constructed and made
'          available to the linker, normal compile and link process proceeds
'          smoothly. Use the include file "fb-dplotlib.bi" for this method.
'
'        - "fb-dplotlibDYN.bi" - the other choice to resolve externals is to
'          dynamically load and resolve the DLL external routines. This process
'          avoids the necessity to construct the the import library and also
'          allows the flexiability of specifying the location of the DPLOTLIB.DLL
'          at run time. Use the include file "fb-dplotlibDYN.bi" for this
'          method (currently setup with hard-coded location for DPLOTLIB.DLL).
'
' ----------------------------------------------------------------------------
' Select one, and ONLY ONE, of the following include files.
'
'  - The file "fb-dplotlib.bi" assumes linking via the import
'    file "libDPLOTLIB.DLL.a" to resolve the calls to routines in DPLOTLIB.DLL.
'
'  - The file "fb-dplotlibDYN.bi" assumes dynamically loading DPLOTLIB.DLL
'    and does not require an import file.
#INCLUDE "fb-dplotlib.bi"
'#INCLUDE "fb-dplotlibDYN.bi"

' Uncomment next line for other (non FreeBASIC) compliers
' using Column Major array storage
'#DEFINE COLMAJ

'$DYNAMIC

Option Explicit
Option Base 0
Const PI As Single = 4# * Atn(1#)

Declare Sub Form_Load()
Declare Sub Command1_Click()
Declare Sub Command2_Click()
Declare Sub Command3_Click()
Declare Sub Command4_Click()
Declare Sub Command5_Click()
Declare Sub Command6_Click()

    Dim Shared d    As DPLOT
    Dim Shared ret  As Long
    Dim Shared X()  As Single
    Dim Shared Y()  As Single
    Dim Shared Z()  As Single

    Dim Kase        As Integer
    Dim LPrmpt      As Integer
    Dim Prmpt$

    Call Form_Load()    ' call initialization "FormLoad" routine

    ' Select a desired case by calling the appropiate "command" btn

    Locate 1,1
    Print "Enter the desired example:"
    Print "  1) DATA XYYY - multiple Y, single X"
    Print "  2) DATA_3D   - 3D Surface plot"
    Print "  3) DATA_3DR  - 3D 'Random' Contour plot"
    Print "  4) DATA_1D   - 1D Grouped data plot"
    Print "  5) DATA_3DR  - Surface Contour w/ Concave Edges"
    Print "  6) DATA_XYYY - Polar Plot"
    Print "    (enter any other number to exit)"

    Prmpt$ = "Enter selection => "
    LPrmpt = Len(Prmpt$)
    Do
        Locate 10,1 : Input "Enter selection => ", Kase

        Select Case Kase
        Case 1
            Call Command1_Click()
            Locate 12,1 : Print "Return Value =";ret
        Case 2
            Call Command2_Click()
            Locate 12,1 : Print "Return Value =";ret
        Case 3
            Call Command3_Click()
            Locate 12,1 : Print "Return Value =";ret
        Case 4
            Call Command4_Click()
            Locate 12,1 : Print "Return Value =";ret
        Case 5
            Call Command5_Click()
            Locate 12,1 : Print "Return Value =";ret
        Case 6
            Call Command6_Click()
            Locate 12,1 : Print "Return Value =";ret
        Case Else
            GoTo Fini
        End Select
        Locate 10,LPrmpt : Print"  "
    Loop

Fini:
End

Private Sub Command1_Click()
    Dim i       As Long
    Dim NP      As Long
    Dim cmds    As String
    Dim DocNum  As Long
    Dim Title1  As String * 79

' Plot sin(PI*x) and cos(PI*x) from x = 0 to 4.
' In this case we've used DATA_XYYY (one X array for one or more Y arrays).
'
' Since the X values are evenly spaced we could also use
' DATA_DXY  - X has only 2 elements, DX and X0
'
' And you can ALWAYS use
' DATA_XYXY - Each curve uses its own X array. This is the only
'             option available if the curves have different X values or
'             a different number of points.

    NP = 1001
    ReDim X(0 To NP - 1) As Single
    ReDim Y(0 To 3 * NP - 1) As Single

    For i = 0 To NP - 1
        X(i)          = (4# * i) / (NP - 1)
        Y(i     )     = Sin(PI * X(i))
        Y(i + NP)     = Cos(PI * X(i))
        Y(i + 2 * NP) = Sin(2#*PI * X(i) + .78)
    Next i

    d.version = DPLOT_DDE_VERSION
    d.DataFormat = DATA_XYYY
    d.MaxCurves = 3         ' Must be >= number of curves we plot
    d.MaxPoints = NP        ' Anything >= NP will do
    d.NumCurves = 3
    d.ScaleCode = SCALE_LINEARX_LINEARY

    ' Placement cordinates of Top Left of Legend Box - percent of plot area
    d.LegendX = 0.05                    ' 5% of width
    d.LegendY = 0.05                    ' 5% of height

    ' NUMBER OF DATA POINTS
    d.NP(0) = NP                        ' Number of data set 1 points
    d.NP(1) = NP                        ' Number of data set 2 points
    d.NP(2) = NP                        ' Number of data set 3 points

    ' LINE TYPES
    d.LineType(0) = LINESTYLE_SOLID
    d.LineType(1) = LINESTYLE_LONGDASH
    d.LineType(2) = LINESTYLE_DASHDOT

    'LEGEND      '"....+....1....+....2....+....3....+....4....+....5....+....6....+....7....+....8"
    d.Legend(0) = "Legend Title"        ' set the Legend Box Title
    d.Legend(1) = "sin({\sp}x)"         ' data set 1 legend
    d.Legend(2) = "cos({\sp}x)"         ' data set 2 legend
    d.Legend(3) = "sin(2{\sp}x+.78)"    ' data set 3 legend

    'DATA LABELS
    d.Label(0)= "Y1"                    ' data set 1 Label (after last point)
    d.Label(1)= "Y2"                    ' data set 2 Label
    d.Label(2)= "Y3"                    ' data set 3 Label

    'CHART TITLE
    d.Title1 = "Data sent to DPlot via DPLOTLIB.DLL"
    d.Title2 = ""
    d.Title3 = ""

    'CHART AXIS
    d.XAxis = "x" '+ Chr$(0)  FreeBASIC internally uses \0 terminated strings
    d.YAxis = "y" '+ Chr$(0)

' The command string can be as complex as you like, limited to 32K characters.
' As an alternative, if your program will be producing many similar plots then
' you might prefer to create a preferences file, edit the file with a text
' editor to remove unwanted entries, and use the [GetPreferences("filename")]
' command.

    cmds =        "[ManualScale(0,-1.25,4,1.25)]"   ' manual scale plot
    cmds = cmds + "[TickInterval(1,0.5,0.25)]"      ' set tick intervals & length
    cmds = cmds + "[Caption(""DPLOTLIB XY Test"")]" ' set caption of window

    ' we define the data legends above in "d" structure
    ' alternatively, can pass within DDE cmd as shown here
''' cmds = cmds + "[Legend(1,""sin({\sp}x)"")]"
''' cmds = cmds + "[Legend(2,""cos({\sp}x)"")]"
''' cmds = cmds + "[Legend(3,""sin(2{\sp}x+.78)"")]"

    cmds = cmds + "[ClearEditFlag()]"
    cmds = cmds + "[DocMaximize()]"                 ' maximize window
    cmds = cmds + "[Size(0,0,0,0)]"                 ' force max size

    DocNum = DPlot_Plot(d, X(0), Y(0), cmds)        ' call the routine

    ret = DocNum
End Sub

Private Sub Command2_Click()

    Dim             As Long i, j ' was "Dim i, j As Long" in vb source (incorrect?)
    Dim Nx          As Long      ' Number of data columns (horizontal grid cells+1)
    Dim Ny          As Long      ' Number of data rows (vertical grid cells+1)
    Dim extents(3)  As Single
    Dim Xi          As Single
    Dim Yi          As Single
    Dim cmds        As String


    Nx = 61
    Ny = 41
    ReDim Z(Nx * Ny - 1) As Single
    extents(0) = -3     ' xlo
    extents(1) = -2     ' ylo
    extents(2) = 3      ' xhi
    extents(3) = 2      ' yhi

    For i = 0 To Nx - 1
        Xi= extents(0) + ((extents(2) - extents(0)) * i) / (Nx - 1)
        For j = 0 To Ny - 1
            Yi= extents(1) + ((extents(3) - extents(1)) * j) / (Ny - 1)
            Z(i * Ny + j) = Sin(Xi) * Cos(Yi)
        Next j
    Next i

    d.version = DPLOT_DDE_VERSION
    d.DataFormat = DATA_3D
    d.MaxCurves = Nx - 1
    d.MaxPoints = Ny - 1
    d.NumCurves = 1
    d.ScaleCode = SCALE_LINEARX_LINEARY
    d.Title1 = "Data sent to DPlot via DPLOTLIB.DLL"
    d.Title2 = "z = sin(x)*cos(y)"
    d.Title3 = ""
    d.XAxis = "x"
    d.YAxis = "y"

    cmds = "[Caption(""DPLOTLIB Contour Test 1"")]"
    cmds = cmds + "[Contour3D(1)][ContourGrid(1)][ContourAxes(1)]"
    cmds = cmds + "[ContourView(30,20)][ContourLevels(20,-1,1)]"
    cmds = cmds + "[ContourScales(1,1,1)]"
    cmds = cmds + "[FontPoints(1,10)][FontPoints(6,10)]"
    cmds = cmds + "[ZAxisLabel(""z"")]"
    cmds = cmds + "[DocMaximize()]"
    cmds = cmds + "[Size(0,0,0,0)]"                 ' force max size

    ret = DPlot_Plot(d, extents(0), Z(0), cmds)

End Sub

Private Sub Command3_Click()
'
'   Sends randomly-placed x,y,z triplets to DPlot and produces a
'   contour plot. The random feature of this example is a bit nonsensical
'   since we're using an equation to generate the data. This DPlot feature
'   is much more useful for truly random data and/or data values that
'   aren't organized on a rectangular grid.
'
'
' **** FreeBASIC - We have to adjust this as FreeBASIC stores array
' **** elements in ROW Major order and not the presummed COLUMN order
'
    Dim i       As Long
    Dim Nz      As Long
    Dim cmds    As String

    Nz = 1000

#IFDEF COLMAJ
    ' Column Major storage
    Dim Node(0 To 2, 0 To Nz - 1) As Single
    For i = 0 To Nz - 1
        Node(0, i) = -PI + 2 * PI * Rnd
        Node(1, i) = -PI + 2 * PI * Rnd
        Node(2, i) = Sin(Node(0, i)) * Cos(Node(1, i))
    Next i
#ELSE
    ' Row Major storage
    Dim Node(0 To Nz - 1, 0 To 2 ) As Single
    For i = 0 To Nz - 1
        Node(i, 0) = -PI + 2 * PI * Rnd
        Node(i, 1) = -PI + 2 * PI * Rnd
        Node(i, 2) = Sin(Node(i, 0)) * Cos(Node(i, 1))
    Next i
#ENDIF

    d.version = DPLOT_DDE_VERSION
    d.DataFormat = DATA_3DR
    d.MaxCurves = 1
    d.MaxPoints = Nz
    d.NumCurves = 1
    d.ScaleCode = SCALE_LINEARX_LINEARY
    d.Title1 = "Data sent to DPlot via DPLOTLIB.DLL"
    d.Title2 = "z = sin(x)*cos(y)"
    d.Title3 = ""
    d.XAxis = "x"
    d.YAxis = "y"

    cmds = "[Caption(""DPLOTLIB Contour Test 2"")]"
    cmds = cmds + "[ContourAxes(1)][ContourGrid(0)][ContourLevels(21,-1,1)]"
    cmds = cmds + "[FontPoints(1,10)][FontPoints(6,10)][DocMaximize()]"
    cmds = cmds + "[Size(0,0,0,0)]"                 ' force max size
    ret = DPlot_Plot(d, 0&, Node(0, 0), cmds)

End Sub

Private Sub Command4_Click()
    Dim         As Long i, j, k   ' was "Dim i, j, k As Long" in vb source
    Dim Ny      As Long
    Dim cmds    As String

    d.version = DPLOT_DDE_VERSION
    d.DataFormat = DATA_1D
    d.MaxCurves = 3
    d.MaxPoints = 80
    d.NumCurves = 3
    d.LegendX = 0.02
    d.LegendY = 0.7
    d.ScaleCode = SCALE_LINEARX_LINEARY
    d.Title1 = "Data sent to DPlot via DPLOTLIB.DLL"
    d.Title2 = ""
    d.Title3 = ""
    d.YAxis = "Amplitude"

    ' Number of points in each group is 50 +/- 15:
    Ny = 0
    For i = 0 To d.NumCurves - 1
        d.NP(i) = 50 + 30 * (Rnd - 0.5)
        Ny = Ny + d.NP(i)
        Mid(d.Legend, (i + 1) * 80 + 1, 80) = "Group " + Chr$(65 + i)
    Next i

    ' Generate group data, centered around 5.0 with progressively larger variations
    ReDim Y(Ny - 1) As Single
    k = 0
    For i = 0 To d.NumCurves - 1
        For j = 0 To d.NP(i) - 1
            Y(k) = 5# + (i + 1) * (-0.5 + Rnd)
            k = k + 1
        Next j
    Next i

    cmds = "[Caption(""DPLOTLIB Box-and-whisker Test"")]"
    cmds = cmds + "[Stat_PlotType(0)][Stat_GroupInfo(15,4)]"
    cmds = cmds + "[Stat_BWOptions(1,1,0,1,1.5,2,3)]"
    cmds = cmds + "[Stat_GrandMean(3,30)]"
    cmds = cmds + "[FontPoints(1,10)][FontPoints(6,10)]"
    cmds = cmds + "[DocMaximize()]"
    cmds = cmds + "[Size(0,0,0,0)]"                 ' force max size
    ret = DPlot_Plot(d, 0&, Y(0), cmds)

End Sub

Private Sub Command5_Click()
'
'   Sends x,y,z triplets defining the outline of a star to DPlot. Initially
'   the resulting surface plot will be convex, in this case a pentagon. We'll
'   then define the border using DPlot_3DBorder so that the plot is
'   star-shaped.
'

' **** FreeBASIC - We have to changed this as FreeBASIC stores array
' **** elements in ROW Major order and not the presummed COLUMN order
'
    Dim         As Double x0, y0, side
    Dim dZero   As Double
    Dim i       As Long
    Dim Nz      As Long
    Dim DocNum  As Long
    Dim ret2    As Long
    Dim cmds    As String

    x0 = 0#     ' Center of the star
    y0 = 0#
    side = 1    ' Distance between star points
#IFDEF COLMAJ
    Dim Star(0 To 2, 0 To 9)      As Double
    Dim Border(0 To 1, 0 To 9)    As Double
    Star(0, 0) = x0 - 0.5 * side
    Star(1, 0) = y0 - 0.5 * side * Tan(54 * PI / 180)
    Star(2, 0) = 0#
    Star(0, 1) = x0
    Star(1, 1) = Star(1, 0) + 0.5 * side * Tan(36 * PI / 180)
    Star(2, 1) = 1#
    Star(0, 2) = x0 + 0.5 * side
    Star(1, 2) = Star(1, 0)
    Star(2, 2) = 0#
    Star(0, 3) = Star(0, 2) - 0.5 * side * (Sin(18 * PI / 180) / Cos(36 * PI / 180))
    Star(1, 3) = Star(1, 2) + 0.5 * side * (Cos(18 * PI / 180) / Cos(36 * PI / 180))
    Star(2, 3) = 1#
    Star(0, 4) = Star(0, 2) + side * Sin(18 * PI / 180)
    Star(1, 4) = Star(1, 2) + side * Cos(18 * PI / 180)
    Star(2, 4) = 0#
    Star(0, 5) = x0 + 0.5 * side * (Sin(18 * PI / 180) / Cos(36 * PI / 180))
    Star(1, 5) = Star(1, 4)
    Star(2, 5) = 1#
    Star(0, 6) = x0
    Star(1, 6) = Star(1, 4) + side * Sin(36 * PI / 180)
    Star(2, 6) = 0#
    Star(0, 7) = 2.0 * x0 - Star(0, 5)
    Star(1, 7) = Star(1, 5)
    Star(2, 7) = 1#
    Star(0, 8) = 2.0 * x0 - Star(0, 4)
    Star(1, 8) = Star(1, 4)
    Star(2, 8) = 0#
    Star(0, 9) = 2.0 * x0 - Star(0, 3)
    Star(1, 9) = Star(1, 3)
    Star(2, 9) = 1#
#ELSE
    Dim Star(0 To 9, 0 To 2)     As Double
    Dim Border(0 To 9, 0 To 1)   As Double
    Star(0, 0) = x0 - 0.5 * side
    Star(0, 1) = y0 - 0.5 * side * Tan(54 * PI / 180)
    Star(0, 2) = 0#
    Star(1, 0) = x0
    Star(1, 1) = Star(0, 1) + 0.5 * side * Tan(36 * PI / 180)
    Star(1, 2) = 1#
    Star(2, 0) = x0 + 0.5 * side
    Star(2, 1) = Star(0, 1)
    Star(2, 2) = 0#
    Star(3, 0) = Star(2, 0) - 0.5 * side * (Sin(18 * PI / 180) / Cos(36 * PI / 180))
    Star(3, 1) = Star(2, 1) + 0.5 * side * (Cos(18 * PI / 180) / Cos(36 * PI / 180))
    Star(3, 2) = 1#
    Star(4, 0) = Star(2, 0) + side * Sin(18 * PI / 180)
    Star(4, 1) = Star(2, 1) + side * Cos(18 * PI / 180)
    Star(4, 2) = 0#
    Star(5, 0) = x0 + 0.5 * side * (Sin(18 * PI / 180) / Cos(36 * PI / 180))
    Star(5, 1) = Star(4, 1)
    Star(5, 2) = 1#
    Star(6, 0) = x0
    Star(6, 1) = Star(4, 1) + side * Sin(36 * PI / 180)
    Star(6, 2) = 0#
    Star(7, 0) = 2.0 * x0 - Star(5, 0)
    Star(7, 1) = Star(5, 1)
    Star(7, 2) = 1#
    Star(8, 0) = 2.0 * x0 - Star(4, 0)
    Star(8, 1) = Star(4, 1)
    Star(8, 2) = 0#
    Star(9, 0) = 2.0 * x0 - Star(3, 0)
    Star(9, 1) = Star(3, 1)
    Star(9, 2) = 1#
#ENDIF

    d.version = DPLOT_DDE_VERSION
    d.DataFormat = DATA_3DR
    d.MaxCurves = 1
    d.MaxPoints = 10
    d.NumCurves = 1
    d.ScaleCode = SCALE_LINEARX_LINEARY
    d.Title1 = "Surface Plot with Concave Edges"
    d.Title2 = "See {\iDefine Boundary} Command on {\iOptions} menu"
    d.Title3 = ""
    d.XAxis = "x"
    d.YAxis = "y"

    cmds = "[Caption(""DPLOTLIB Surface Plot w/ Concave Edges"")]"
    cmds = cmds + "[ContourAxes(1)][ContourGrid(0)][ContourMethod(0)][ContourLevels(20,0,1)]"
    cmds = cmds + "[FontPoints(1,10)][FontPoints(6,10)][DocMaximize()]"
    cmds = cmds + "[Size(0,0,0,0)]"                 ' force max size
    dZero = 0.0
    DocNum = DPlot_Plot8(d, dZero, Star(0, 0), cmds)
    ret = DocNum
    If DocNum > 0 Then
       For i = 0 To 9
#IFDEF COLMAJ
          Border(0, i) = Star(0, i)
          Border(1, i) = Star(1, i)
#ELSE
          Border(i, 0) = Star(i, 0)
          Border(i, 1) = Star(i, 1)
#ENDIF
       Next i
       ret2 = DPlot_3DBorder(DocNum, 10, Border(0, 0))
    End If

End Sub
Private Sub Command6_Click()

    Dim i       As Long
    Dim NP      As Long
    Dim DocNum  As Long
    Dim cmds    As String

' Plot sin(3*x) from x = 0 to 2PI
' In this case we've used DATA_XYYY (one X array for one or more Y arrays).
'
' Since the X values are evenly spaced we could also use
' DATA_DXY  - X has only 2 elements, DX and X0
'
' And you can ALWAYS use
' DATA_XYXY - Each curve uses its own X array. This is the only
'             option available if the curves have different X values or
'             a different number of points.

    NP = 1001
    ReDim X(0 To NP - 1) As Single
    ReDim Y(0 To NP - 1) As Single

    For i = 0 To NP - 1
        X(i) = (2# * PI * i) / (NP - 1)
        Y(i) = Abs(Sin(3# * X(i)))
    Next i

    d.version = DPLOT_DDE_VERSION
    d.DataFormat = DATA_XYYY
    d.MaxCurves = 1         ' Must be >= number of curves we plot
    d.MaxPoints = NP        ' Anything >= NP will do
    d.NumCurves = 1
    d.ScaleCode = SCALE_POLAR
    d.NP(0) = NP
    d.LineType(0) = LINESTYLE_SOLID
    d.Title1 = "abs(sin(3{\sf}))"
    d.Title2 = ""
    d.Title3 = ""

    cmds = "[ManualScale(0,0,0,1.2)]"                 ' X limits are ignored
    cmds = cmds + "[TickInterval(2,30,0.2)]"            ' 1st "2" says X interval is in degrees
    cmds = cmds + "[Caption(""DPLOTLIB XY Test"")]"
    cmds = cmds + "[PolarPlotStyle(7)]"
    cmds = cmds + "[NumberFormat(0,8)]"                 ' PI Fractions
    cmds = cmds + "[ClearEditFlag()][DocMaximize()]"
    cmds = cmds + "[Size(0,0,0,0)]"                 ' force max size
    DocNum = DPlot_Plot(d, X(0), Y(0), cmds)
    ret = DocNum

End Sub

Private Sub Form_Load()
    DPlot_SetErrorMethod (2)
End Sub
